########################################################################################
# (Note: hard tab setting for this source file is: 4)
#
# STLFilt.pl: STL error message decryption filter (A Perl script) supporting
#			  Microsoft VC++6 when working with any of the following libraries:
#					Native MSVC 6 Library
#					Dinkumware 3.08 Library
#					STLPort 4.0/4.1b? Library
#
# Copyright (C) 2001 by Leor Zolman (leor@bdsoft.com, www.bdsoft.com)
# Permission is granted to use this code without restriction
# provided this copyright notice is included in the code.
#
# Release 0.992: 5/17/2001
#
# This file is part of a package including:
#	CL.CPP:			The "Proxy" CL.EXE replacement,
#	STLTask			A taskbar icon-based error decryption control utility,
#	STLFilt.BAT		A "low-bandwith" batch file for error decryption control
#
# For installation instructions, see INSTALL.txt
# For General Info, Change log, etc., see README.txt
#
# Purpose:	Transform STL template-based typenames into a minimal,
#			*readable* format. We might lose some details...but retain our sanity!
#
# Command line arguments:
#			/iter:x (or) -iter:x         Set iterator policy to x,
#										 where x is s[mall], m[edium] or l[ong]
#										 (typically conveyed by *my* CL.EXE)
#
# Acknowledgements:
# -----------------
# Thanks to Dave Smallberg for the idea (wish I'd have thought of it myself)
# and to Scott Meyers for putting on his ESTL seminar -- the event that
# inspired this project. 
#########################################################################################

###########################################################################################
# User-conifgurable options:
#
$newiter = 'iter';			# ('iter','IT' or...) shorten the word "iterator" to this

							# default iterator policy:
$def_iter_policy = 'M';		# 'L' (Long): NEVER remove iterator type name qualification
							# 'M' (Medium): USUALLY remove iterator type name qualification
							#               leave intact when iter type may be significant
							#				to the diagnostic
							# 'S' (Short): ALWAYS remove iterator type name qualification

$alloc = 'alloc';			# shorten the word "allocator" to this

$n_passes = 4;				# number of passes to make (thus supporting at least $n_passes
							#	levels of template nesting)
#
# END of user-configurable options
###########################################################################################

$iter_policy = $def_iter_policy;				# default iterator policy

if ($ARGV[0] =~ /^[\/-]iter:([SML])[a-zA-Z]*$/i) # allow command-line iter_policy
{												 # specification of form: /iter:x
	$iter_policy = "\u$1";
	shift;
}

$id = '[a-zA-Z_]\w*';							# pattern for a simple identifier
$p = '(?: \*){0,5}';							# suffix for "ptr", "ptr to ptr", "ptr to ptr to ptr", ad nauseum.
$idp = "(?:$id )?(?:$id )?(?:$id )?$id ?$p ?";	# one or more identifiers with perhaps some *'s after

												# simple id or basic template spec
$cid = "(?:$id<$idp(?: const)?,$idp(?: const)?>$p|$id<$idp(?: const)?>$p|$idp(?: const)?) ?";

												# one more level, and we finally get to a "type"
$t = "(?:$id<$id<$cid>$p ?>$p|$id<$cid,$cid(?:,$cid)?>|$id<$cid>$p|$cid)";


while (<>)
{
# eliminate some noise:
  s/(\W)std\:\:/\1/g;

  s/(\W)class /\1/g;
  s/(\W)struct /\1/g;
  s/(\W)__thiscall /\1/g;
  s/\(__cdecl \*\)//g;
  s/(\W)__cdecl\W/\1/g;
  s/(\W)_STL::/\1/g;				# for STLPort

# simplify the ubiquitous "vanilla" string, cin, cout (w/optional default allocator):
  s/_?basic_(string|istream|ostream)<char,char_traits<char>(,_?allocator<char>)? ?>/\1/g;
  s/_?basic_(string|istream|ostream)<unsigned short,char_traits<unsigned short>(,_?allocator<unsigned short>)? ?>/w\1/g;
  s/_?basic_(string|istream|ostream)<_E(lem)?,_Tr(aits)?,_A(lloc)?>/\1/g;

  for ($pass = 1; $pass <= $n_passes; $pass++)	# each pass processes an additional level of nesting
  {
	s/([io])stream_iterator<($t),char,char_traits<char> *(,$t)?>/\1stream_iterator<\2>/g;

	# delete allocators from template typenames completely:
	s/,? ?allocator<$t ?> ?>/>/g;
	s/,? ?allocator<$t ?>(,[01]) ?>/\1>/g;

	# delete the allocator specification from parameter lists, *unless* the message
	# mentions an error converting *to* the allocator type (OK, OK, this is ugly... but the
	# only time we really want to think about the allocator is when an error message
	# refers to it...)

	unless (/' to '.*allocator/)
	{
	  s/,allocator<$t ?> ?//g;				# the leading comma allows the full spec.
	  s/,const allocator<$t ?> ?&//g;			# to appear in the error message details
	}

	# reduce any remaining allocator parameters to just "$alloc":
	s/allocator<$t ?> ?([\)'])/$alloc\1/g;
	s/const allocator<$t ?> &([\)'])/const $alloc\1/g;


# This, BTW, is how I debug this thing...stick one of these before suspect code to see progressive x'formations
# (The "list" below would be replaced by a known pattern in the construct i'm debugging)
#/list/ && print "-----(Pass $pass)------------------------DEBUG:\n$_\n";

	# STLPort iterators:
	s/_List_iterator<($t),_Nonconst_traits<\1 ?> ?>/list<\1>::iterator/g;
	s/_List_iterator<($t),_Const_traits<\1 ?> ?>/list<\1>::const_iterator/g;
	s/_Deque_iterator<($t),_Nonconst_traits<\1 ?> ?>/deque<\1>::iterator/g;
	s/_Deque_iterator<($t),_Const_traits<\1 ?> ?>/deque<\1>::const_iterator/g;

	# STLPort set/multiset:
	# Since STLPort uses the same iterator type for both set and multiset, we just
	# say "gen_set<T>::iterator" to mean the "GENERIC" set/multiset iterator type:
	s/_Rb_tree_iterator<($t),_Nonconst_traits<\1 ?> ?>/gen_set<\1>::iterator/g;
	s/_Rb_tree_iterator<($t),_Const_traits<\1 ?> ?>/gen_set<\1>::const_iterator/g;

	# STLPort map/multimap (same "GENERIC" iterator approach as above):
	s/_Rb_tree_iterator<pair<($t) const ?,($t)>,_Nonconst_traits<pair<\1 const ?,\2> > >/gen_map<\1,\2>::iterator/g;
	s/_Rb_tree_iterator<pair<($t) const ?,($t)>,_Const_traits<pair<\1 const ?,\2> > >/gen_map<\1,\2>::const_iterator/g;

	# Dinkumware map/multimap:
	s/(multi)?map<($t),($t)(,$t(<\2 ?>)?)? ?>/\1map<\2,\3\4>/g;
	s/_Tree<_Tmap_traits<($t),($t)(,$t(<\1 ?>)?)?,0 ?> ?>/map<\1,\2\3>/g;
	s/_Tree<_Tmap_traits<($t),($t)(,$t(<\1 ?>)?)?,1 ?> ?>/multimap<\1,\2\3>/g;

	# native VC++ map/multimap:
	s/_Tree<($t),pair<\1 const ?,($t) ?>,(multi)?map<\1,\2(,$t(<\1 ?>)?)? ?>\:\:_Kfn\4? ?>/\3map<\1,\2>/g;
	s/\(const pair<($t) const ?,($t) ?> \*,const pair<\1 const ?,\2 ?> \*,const $t<\1 ?> & ?&\)/($newiter, $newiter, \3<\1>)/g;

	# map/multimap members common to native/Dinkum:
	s/const pair<($t) const ?,($t) ?> ([\*&])/pair<\1,\2> \3/g;

	# native VC++ set/multiset
 	s/_Tree<($t),\1,(multi)?set<\1(,$t(?:<\1 ?>)?)? ?(,$t)?>\:\:_Kfn(?:\3(?:<\1 ?>)?)? ?(,\4)?>/\2set<\1\3>/g;

	# Dinkumware set/multiset:
	s/_Tree<_Tset_traits<($t)(,$t)?,0 ?> ?>/set<\1\2>/g;
	s/_Tree<_Tset_traits<($t)(,$t)?,1 ?> ?>/multiset<\1\2>/g;

	# simplify default, ubiquitous "less" functors, but leave others intact:
	s/,_?less<$t ?>//g;
	s/,Comp<$t ?>//g;		# STLPort's default comparison function
	s/(,)?const less<$t ?> &/\1less &/g;
  }


# reduce iterators according to $iter_policy:

  if ($iter_policy eq 'M')					# policy 'M': USUALLY remove:
  {
	unless (/( of type|' to '|from ')$t\:\:(const_)?iterator/  # Shorten to $newiter and
			|| /iterator' does/)							   # *remove* the base type completely...
	{														   # as long as the error message doesn't	
		s/$t\:\:(const_)?iterator/\1$newiter/g;				   # mention iterators!				
	}			
  }
  elsif ($iter_policy eq 'S')				# policy 'S': ALWAYS remove:
  {
	s/$t\:\:iterator/$newiter/g;			#	remove the base type completely
  }
											# All policies (including 'L'):
  s/iterator/$newiter/g;					#	shorten "iterator" to "$newiter", 


# reduce "double" constructor names 'T::T' to just 'T':

  s/'string\:\:string([\('])/'string\1/;
  s/'(.*)\:\:\1([\('])/'\1\2/;

# I'm sorry, Microsoft needs grammar lessons:
  s/take 1 parameters/take 1 parameter/g;

# throw away those obnoxious "long identifier" warnings, and subsequent references
# to those warnings (eliminates the need for the #pragma hack):
  /truncated to '255'/ && next;
  /see reference to template/ && next;

# get rid of that useless space between asterisks in ptrs-to-ptrs:
  s/ \* \* \* \*/ ****/g;
  s/ \* \* \*/ ***/g;
  s/ \* \*/ **/g;

# deal with some other non-critical (and often not even very aesthetic) spaces:
  s/ >/>/g;
  s/ ,/,/g;

# and FINALLY, print out the result of all transformations:

  print;

}

exit 0;			# CL.CPP will take care of returning the appropriate status to caller

