/*******************************************************************************
  File: XdrStream.cpp
  Copyright (c) 1999 Bleading Edge Software Technologies, Inc.

  This Software is distributed under the open source, free software model.
  Permission to use, modify, and distribute this Software, including
  incorporating it into proprietary software, is granted without fee,
  provided this copyright notice appear in all copies of the code.
  You are under no obligation to distribute source code which is derived 
from
  or uses this Software. You may not do anything however, to prevent the
  continued distribution of this Software under an open source distribution
  model. This includes, but is not limited to, such things as copyrighting
  or claiming authorship of the Software.
  This Software is distributed in the hope that it will be useful, but it is
  provide "as is", WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

//$-----------------------------------------------------------------------------
$Header:$
  Author: Jack W. Reeves

//!----------------------------------------------------------------------------|
Description:
  Implements the oXDR_Stream and the XDR_Stream

//#----------------------------------------------------------------------------|
Notes:

//*****************************************************************************/

#include "XDRStream.h"

#include "platform.h"


namespace Util {


oXDR_Stream::oXDR_Stream(XDR_Streambuf* sb)
  : inherited(sb)
{
	// empty
}


oXDR_Stream::~oXDR_Stream()
{
	// empty
}


oXDR_Stream&
oXDR_Stream::operator<<(oXDR_Stream& (*pf)(oXDR_Stream&))
{
	pf(*this);
	return *this;
}


oXDR_Stream&
oXDR_Stream::operator<<(std::basic_ios<XDR_Char>& 
(*pf)(std::basic_ios<XDR_Char>&))
{
	pf(*this);
	return *this;
}


oXDR_Stream&
oXDR_Stream::operator<<(std::ios_base& (*pf)(std::ios_base&))
{
	pf(*this);
	return *this;
}


oXDR_Stream&
oXDR_Stream::operator<<(bool n)
{
	return operator<<(static_cast<long>(n ? 1 : 0));
}


oXDR_Stream&
oXDR_Stream::operator<<(char n)
{
	return operator<<(static_cast<long>( static_cast<signed char>(n) ));
}


oXDR_Stream&
oXDR_Stream::operator<<(signed char n)
{
	return operator<<(static_cast<long>(n));
}


oXDR_Stream&
oXDR_Stream::operator<<(unsigned char n)
{
	return operator<<(static_cast<unsigned long>(n));
}


oXDR_Stream&
oXDR_Stream::operator<<(short n)
{
	return operator<<(static_cast<long>(n));
}


oXDR_Stream&
oXDR_Stream::operator<<(unsigned short n)
{
	return operator<<(static_cast<unsigned long>(n));
}


oXDR_Stream&
oXDR_Stream::operator<<(int n)
{
	return operator<<(static_cast<long>(n));
}


oXDR_Stream&
oXDR_Stream::operator<<(unsigned int n)
{
	return operator<<(static_cast<unsigned long>(n));
}


oXDR_Stream&
oXDR_Stream::operator<<(long n)
{
	XDR_Char c;
	::memcpy(c.data, &n, sizeof(c.data));
#ifdef LITTLE_ENDIAN
	c.swap_bytes();
#endif
	put(c);
	return *this;
}


oXDR_Stream&
oXDR_Stream::operator<<(unsigned long n)
{
	XDR_Char c;
	::memcpy(c.data, &n, sizeof(c.data));
#ifdef LITTLE_ENDIAN
	c.swap_bytes();
#endif
	put(c);
	return *this;
}


oXDR_Stream&
oXDR_Stream::operator<<(float f)
{
	XDR_Char c;
	::memcpy(c.data, &f, sizeof(c.data));
#ifdef LITTLE_ENDIAN
	c.swap_bytes();
#endif
	put(c);
	return *this;
}


oXDR_Stream&
oXDR_Stream::operator<<(double f)
{
	XDR_Char c[2];
	::memcpy(c, &f, sizeof(f));
#ifdef LITTLE_ENDIAN
	std::swap(c[0], c[1]);
	c[0].swap_bytes();
	c[1].swap_bytes();
#endif
	write(c, 2);
	return *this;
}


// opaque data inserters
oXDR_Stream&
oXDR_Stream::put(const void* d, size_t l)
{
	XDR_Char c;
	const unsigned char* data = reinterpret_cast<const unsigned char*>(d);
	size_t k = l / 4;
	while (k > 0) {
		::memcpy(c.data, data, sizeof(c.data));
		data += sizeof(c.data);
		--k;
		put(c);
	}

	int p = l % 4;
	if (p) {
		::memcpy(c.data, data, 4 - p);
		c.pad(p);
		put(c);
	}
	return *this;
}


oXDR_Stream&
oXDR_Stream::vput(const void* d, size_t l)
{
	*this << static_cast<unsigned long>(l);
	return put(d, l);
}


oXDR_Stream&
oXDR_Stream::vput(const std::vector<unsigned char>& d)
{
	return vput(&d[0], d.size());
}


template<typename T>
oXDR_Stream&
oXDR_Stream::put_arr(const T* a, size_t l)
{
	while (l > 0) {
		*this << *a;
		++a; --l;
	}
	return *this;
}


template<typename T>
oXDR_Stream&
oXDR_Stream::vput_arr(const T* a, size_t l)
{
	*this << static_cast<unsigned long>(l);
	return put_arr(a, l);
}


template<typename T>
oXDR_Stream&
oXDR_Stream::vput_arr(const std::vector<T>& v)
{
	return vput_arr(&v[0], v.size());
}


oXDR_Stream&
oXDR_Stream::put(char_type c)
{
	rdbuf()->sputc(c);	// TBS - error checking
	return *this;
}


oXDR_Stream&
oXDR_Stream::write(const char_type* s, std::streamsize n)
{
	rdbuf()->sputn(s, n);	// TBD - error checking
	return *this;
}


oXDR_Stream&
oXDR_Stream::flush()
{
	rdbuf()->pubsync();
	return *this;
}


oXDR_Stream&
operator<<(oXDR_Stream& os, const char* s)
{
	// convert the string to ASCII as necessary
	return os.vput(s, ::strlen(s));
}


oXDR_Stream&
operator<<(oXDR_Stream& os, const signed char* s)
{
	return os << reinterpret_cast<const char*>(s);
}


oXDR_Stream&
operator<<(oXDR_Stream& os, const unsigned char* s)
{
	return os << reinterpret_cast<const char*>(s);
}


oXDR_Stream&
operator<<(oXDR_Stream& os, const std::string& str)
{
	// convert the string to ASCII as necessary
	return os.vput(str.data(), str.size());
}

oXDR_Stream&
flush(oXDR_Stream& os)
{
	return os.flush();
}


oXDR_Stream::sentry::sentry(oXDR_Stream& os)
  : _ok(os.good())
{
	// empty
}


oXDR_Stream::sentry::~sentry()
{
	// empty
}


//
// XDR_Stream
//
XDR_Stream::XDR_Stream(XDR_Streambuf* sb)
  : iXDR_Stream(sb),
    oXDR_Stream(sb)
{
	// empty
}


XDR_Stream::~XDR_Stream()
{
	// empty
}

/***  Currently not implemented
oXDR_Stream&
oXDR_Stream::operator<<(long double f);


oXDR_Stream&
oXDR_Stream::operator<<(XDR_Streambuf* sb);


oXDR_Stream::pos_type
oXDR_Stream::tellp();


oXDR_Stream&
oXDR_Stream::seekp(pos_type);


oXDR_Stream&
oXDR_Stream::seekp(off_type, ios_base::seekdir);
***/

} //> namespace Util


