#ifndef cdc_h__
#define cdc_h__

typedef struct cdc cdc_t;

#include "cdc_blk.h"
#include "cdc_bat.h"
#include "cdc_map.h"
#include "cdc_data.h"
#include "cdc_env.h"

/*
 * minimum number of bytes needed for basic header 
 *    + block header
 */
#define CDC_HEADER_SIZE 64

#define BAT_ENTRY_EOL  -1
#define MAP_FLAG_COMPRESSED 0x80000000 /* entry is compressed */

#define U32_PUT(x, y) \
  *((x)++) = (unsigned char) ((y) >> 24); \
  *((x)++) = (unsigned char) ((y) >> 16); \
  *((x)++) = (unsigned char) ((y) >>  8); \
  *((x)++) = (unsigned char) ((y)      )

#define U32_GET(y, x) \
  y = (((unsigned char) ((x)[0])) << 24UL) \
  | (((unsigned char) ((x)[1])) << 16UL)   \
  | (((unsigned char) ((x)[2])) <<  8UL)   \
  | (((unsigned char) ((x)[3])));          \
  (x) += 4


struct cdc
{
  char            *filename;
  comprio_env_t  *env;
  struct
  {
    unsigned long  magic;       /* file magic #                 */
    unsigned long  version;     /* file version                 */
    unsigned long  file_size;   /* # of bytes virtual file      */
    unsigned long  blocks;      /* # of blocks in the header    */
    unsigned long  dirty;       /* non-0 if the file is corrupt */
    unsigned char *buffer;      /* header buffer                */
  } hdr;
  int            fd;            /* file descriptor                    */
  unsigned long  offset;        /* byte offset of next read or write  */
  unsigned long  map_distance;  /* # of blocks between two map tables */

  cdc_blk_t      blk;  /* block info    */
  cdc_bat_t      bat;  /* bat info      */
  cdc_map_t      map;  /* map info      */
  cdc_data_t     data; /* data info     */
};

int cdc_init(cdc_t *cdc);
int cdc_hdr_flush(cdc_t *cdc, int complete);
int cdc_hdr_read(cdc_t *cdc);
int cdc_hdr_dirty(cdc_t *cdc);
int cdc_missing_tbl_create(cdc_t *cdc, unsigned long tbl_no, int is_bat);

#endif /* cdc_h__ */

