#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <unistd.h>
#include <fcntl.h>

#include "comprio.h"
#include "cdc_env.h"
#include "cdc_zlib.h"
/*
 * the generic environment
 */
comprio_env_t cdc_base_env = 
{
  512, /* block size */
   64, /* blocks / cluster */
    1, /* blocks / map     */
   16, /* blocks / bat     */
  open,
  close,
  read,
  write,
  lseek,
  fsync,
  fstat,
  {&cdc_base_env, cdc_zlib_deflate_init, cdc_zlib_deflate_cleanup, cdc_zlib_deflate},
  {&cdc_base_env, cdc_zlib_inflate_init, cdc_zlib_inflate_cleanup, cdc_zlib_inflate}
};

int comprio_env_alloc(comprio_env_t **dst)
{
  int            rc;
  comprio_env_t *env;

  env = malloc(sizeof(*env));
  if (0 == env)
  {
    rc = ENOMEM;
  } else
  {
    *env = cdc_base_env;
    env->deflate.arg = env;
    env->inflate.arg = env;
    *dst = env;
    rc = 0;
  }
  return rc;
}

int comprio_env_free(comprio_env_t *env)
{
  free(env);
  return 0;
}
  
int comprio_env_block_size_set(comprio_env_t *env, size_t sz)
{
  env->blk_size = sz;
  return 0;
}

int comprio_env_cluster_size_set(comprio_env_t *env, size_t ct)
{
  env->blk_per_cluster = ct;
  return 0;
}

int comprio_env_map_size_set(comprio_env_t *env, size_t ct)
{
  env->blk_per_map = ct;
  return 0;
}

int comprio_env_bat_size_set(comprio_env_t *env, size_t ct)
{
  env->blk_per_bat = ct;
  return 0;
}

int comprio_env_fn_open_set(
  comprio_env_t *env,
  int (*fn)(const char *, int, ...)
  )
{
  env->open = fn;
  return 0;
}
  
int comprio_env_fn_close_set(
  comprio_env_t *env,
  int (*fn)(int)
  )
{
  env->close = fn;
  return 0;
}

int comprio_env_fn_read_set(
  comprio_env_t *env,
  ssize_t (*fn)(int, void *, size_t)
  )
{
  env->read = fn;
  return 0;
}

int comprio_env_fn_write_set(
  comprio_env_t *env,
  ssize_t (*fn)(int, const void *, size_t)
  )
{
  env->write = fn;
  return 0;
}

int comprio_env_fn_seek_set(
  comprio_env_t *env,
  off_t (*fn)(int, off_t, int)
  )
{
  env->seek = fn;
  return 0;
}

int comprio_env_fn_sync_set(
  comprio_env_t *env,
  int (*fn)(int)
  )
{
  env->sync = fn;
  return 0;
}

int comprio_env_fn_stat_set(
  comprio_env_t *env,
  int (*fn)(int, struct stat *)
  )
{
  env->stat = fn;
  return 0;
}
int comprio_env_deflate_arg_set(
  comprio_env_t *env,
  void *arg
  )
{
  env->deflate.arg = arg;
  return 0;
}

int comprio_env_deflate_init_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  )
{
  env->deflate.init = fn;
  return 0;
}

int comprio_env_deflate_cleanup_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  )
{
  env->deflate.cleanup = fn;
  return 0;
}

int comprio_env_deflate_set(
  comprio_env_t *env,
  int (*fn)(
    void       *arg, 
    void       *dst, 
    size_t     *dst_sz, 
    const void *src, 
    size_t      src_sz
    )
  )
{
  env->deflate.fn = fn;
  return 0;
}

int comprio_env_inflate_arg_set(
  comprio_env_t *env,
  void *arg
  )
{
  env->inflate.arg = arg;
  return 0;
}

int comprio_env_inflate_init_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  )
{
  env->inflate.init = fn;
  return 0;
}

int comprio_env_inflate_cleanup_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  )
{
  env->inflate.cleanup = fn;
  return 0;
}

int comprio_env_inflate_set(
  comprio_env_t *env,
  int (*fn)(
    void       *arg, 
    void       *dst, 
    size_t     *dst_sz, 
    const void *src, 
    size_t     src_sz
    )
  )
{
  env->inflate.fn = fn;
  return 0;
}


