#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <errno.h>

#include <zlib.h>
#include "comprio.h"
#include "cdc_zlib.h"

typedef struct cdc_zlib
{
  comprio_env_t *env;  /* owning environment */
  z_stream       zstr; /* stream             */
} cdc_zlib_t;

  
int cdc_zlib_inflate_init(void *arg)
{
  cdc_zlib_t *zt;
  int         rc;

  zt = calloc(1, sizeof(*zt));
  if (0 == zt)
  {
    rc = ENOMEM;
  } else
  {
    zt->env = arg;
    rc = inflateInit(&zt->zstr);
    if (Z_OK == rc)
    {
      rc = 0;
      zt->env = arg;
      comprio_env_inflate_arg_set(arg, zt);
    } else
    {
      switch (rc)
      {
        case Z_MEM_ERROR: rc = ENOMEM; break;
        default:          rc = EINVAL; break;
      }
      free(zt);
    }
  }
  return rc;
}

int cdc_zlib_inflate_cleanup(void *arg)
{
  int         rc;
  cdc_zlib_t *zt;

  zt = arg;
  rc = inflateEnd(&zt->zstr);
  assert(0 == rc);
  comprio_env_inflate_arg_set(zt->env, zt->env);
  free(zt);
  return 0;
}

int cdc_zlib_inflate(
  void       *arg,
  void       *dst,
  size_t     *dst_sz,
  const void *src,
  size_t      src_sz
  )
{
  int         rc;
  cdc_zlib_t *zt;

  zt = arg;
  zt->zstr.next_in   =  src;
  zt->zstr.avail_in  =  src_sz;
  zt->zstr.next_out  =  dst;
  zt->zstr.avail_out = *dst_sz;
  rc = inflate(&zt->zstr, Z_FINISH);
  assert(Z_STREAM_END == rc);
  assert(zt->zstr.avail_out == 0);
  rc = inflateReset(&zt->zstr);
  assert(Z_OK == rc);
  rc = 0;
  return rc;
}

int cdc_zlib_deflate_init(void *arg)
{
  cdc_zlib_t *zt;
  int         rc;

  zt = calloc(1, sizeof(*zt));
  if (0 == zt)
  {
    rc = ENOMEM;
  } else
  {
    rc = deflateInit(&zt->zstr, Z_BEST_SPEED);
    if (Z_OK == rc)
    {
      zt->env = arg;
      comprio_env_deflate_arg_set(arg, zt);
      rc = 0;
    } else
    {
      switch (rc)
      {
        case Z_MEM_ERROR: rc = ENOMEM; break;
        default:          rc = EINVAL; break;
      }
      free(zt);
    }
  }
  return rc;
}

int cdc_zlib_deflate_cleanup(void *arg)
{
  int         rc;
  cdc_zlib_t *zt;

  zt = arg;
  rc = deflateEnd(&zt->zstr);
  assert(0 == rc);
  comprio_env_deflate_arg_set(zt->env, zt->env);
  free(zt);
  return 0;
}

int cdc_zlib_deflate(
  void       *arg,
  void       *dst,
  size_t     *dst_sz,
  const void *src,
  size_t      src_sz
  )
{
  int         rc;
  cdc_zlib_t *zt;

  zt = arg;
  zt->zstr.next_in   =  src;
  zt->zstr.avail_in  =  src_sz;
  zt->zstr.next_out  =  dst;
  zt->zstr.avail_out = *dst_sz;
  rc = deflate(&zt->zstr, Z_FINISH);
  if (Z_STREAM_END != rc)
  {
    rc = E2BIG;
    deflateReset(&zt->zstr);
  } else
  {
    *dst_sz -= zt->zstr.avail_out;
    rc = 0;
  }
  return rc;
}

