#ifndef comprio_h__
#define comprio_h__

#include <sys/stat.h>

/*
 * version history:
 *   0x00010000 -- first version
 *   0x00010001 -- incompatible: added ``dirty'' flag to header
 *   0x00010002 -- incompatible: added first-free pointer to BAT header
 */
#define COMPRIO_MAGIC   0x12345678
#define COMPRIO_VERSION 0x00010002

typedef struct cdc_env comprio_env_t;

typedef struct comprio_counter
{
  unsigned long read;
  unsigned long write;
} comprio_counter_t;

typedef struct comprio_stat
{
  struct
  {
    comprio_counter_t counters;
    unsigned long     size;
    unsigned long     blocks;
  } blk;
  struct
  {
    comprio_counter_t counters;
    unsigned long     size;
    unsigned long     blocks;
  } data;
  struct
  {
    comprio_counter_t counters;
    unsigned long     blocks; /* blocks / bat */
    unsigned long     count;  /* # of bats    */
  } bat;
  struct
  {
    comprio_counter_t counters;
    unsigned long     blocks; /* blocks / map table */
    unsigned long     count;  /* # of map tables    */
  } map;
} comprio_stat_t;

/*
 * environment functions
 */
int     comprio_env_alloc(comprio_env_t **env);
int     comprio_env_free(comprio_env_t *env);
int     comprio_env_block_size_set(comprio_env_t *env, size_t sz);
int     comprio_env_cluster_size_set(comprio_env_t *env, size_t ct);
int     comprio_env_map_size_set(comprio_env_t *env, size_t ct);
int     comprio_env_bat_size_set(comprio_env_t *env, size_t ct);
int     comprio_env_fn_open_set(
  comprio_env_t *env,
  int (*fn)(const char *, int, ...)
  );
int     comprio_env_fn_close_set(
  comprio_env_t *env,
  int (*fn)(int)
  );
int     comprio_env_fn_read_set(
  comprio_env_t *env,
  ssize_t (*fn)(int, void *, size_t)
  );
int     comprio_env_fn_write_set(
  comprio_env_t *env,
  ssize_t (*fn)(int, const void *, size_t)
  );
int     comprio_env_fn_seek_set(
  comprio_env_t *env,
  off_t (*fn)(int, off_t, int)
  );
int     comprio_env_fn_sync_set(
  comprio_env_t *env,
  int (*fn)(int)
  );
int     comprio_env_fn_stat_set(
  comprio_env_t *env,
  int (*fn)(int, struct stat *)
  );
int     comprio_env_deflate_arg_set(
  comprio_env_t *env,
  void *arg
  );
int     comprio_env_deflate_init_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  );
int     comprio_env_deflate_cleanup_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  );
int     comprio_env_deflate_set(
  comprio_env_t *env,
  int (*fn)(
    void       *arg, 
    void       *dst, 
    size_t     *dst_sz, 
    const void *src, 
    size_t      src_sz
    )
  );
int    comprio_env_inflate_arg_set(
  comprio_env_t *env,
  void *arg
  );
int     comprio_env_inflate_init_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  );
int     comprio_env_inflate_cleanup_set(
  comprio_env_t *env,
  int (*fn)(void *arg)
  );
int     comprio_env_inflate_set(
  comprio_env_t *env,
  int (*fn)(
    void       *arg, 
    void       *dst, 
    size_t     *dst_sz, 
    const void *src, 
    size_t      src_sz
    )
  );
  
/*
 * i/o functions
 */
 
int     comprio_open(comprio_env_t *env, const char *fname, int flags, ...);
int     comprio_close(int fd);
int     comprio_sync(int fd);
ssize_t comprio_read(int fd, void *buf, size_t count);
ssize_t comprio_write(int fd, const void *buf, size_t count);
off_t   comprio_seek(int fd, off_t offset, int whence);
int     comprio_stat(int fd, struct stat *buf);
int     comprio_counter(int fd, comprio_stat_t *buf);
int     comprio_counter_dump(int fd, FILE *out);
int     comprio_lock(int fd, int type);
#endif /* comprio_h__ */

