// base.h
// Copyright 2001, Jim Hyslop, all rights reserved.
//
// Declaration of the base class to be stored in the factory.

#ifndef BASE_HEADER_DEFINED
#define BASE_HEADER_DEFINED

#include <iosfwd>
#include <string>

// The class to be stored in the factory template.
class Base
{
public:
    typedef std::string BASE_KEY_TYPE;
private:
    // Derived classes must override this function, and
    // provide their own classID value.
    virtual BASE_KEY_TYPE classID() const;

    // The class's data to be stored and retrieved.
    int myValue;

protected:
    // Derived classes must call their parent's implementation
    // of this function after they load themselves.
    virtual void do_read(std::istream &);

    // Derived classes must call their parent's implementation
    // of this function after they save themselves.
    virtual void do_write(std::ostream &) const;

    // Derived classes must call their parent's implementation
    // of this function.
    virtual void initialize();

public:
    // This virtual function is used to illustrate the dynamic binding.
    virtual void displayValues();

    // Streaming functions.
    void read(std::istream &);
    void write(std::ostream &) const;

    // Two stage initialization - required for this implementation of the
    // Factory Method pattern.
    void init();

    virtual ~Base();
};

// Streaming helpers.  Note that they are not friends!
std::ostream & operator <<(std::ostream &, const Base &);
std::istream & operator >>(std::istream &, Base &);

#endif // sentry
