// file: derived.cpp
// Implementation of derived class.

#ifdef _MSC_VER
// Disable MS warnings about long identifiers
#pragma warning(disable:4786)
#endif

// Rule: first file included should be the one containing the class
// definition (See "Large Scale C++ programming", John Lakos)
#include "derived.h" 
#include "factory.h"
#include <iostream>

// Register this class in the factory (see June "Conversations" column)
const Base::BASE_KEY_TYPE derivedName ("Derived");
static registerInFactory<Base, Derived, Base::BASE_KEY_TYPE> registerMe(derivedName);

Base::BASE_KEY_TYPE Derived::classID() const
{
    return derivedName;
}

// Virtual overriders
void Derived::initialize()
{
    // Initialize our own data (obviously this would be more substantial
    // and include error checking in a Real World Application).
    std::cout << "Enter a word: ";
    std::cin >> mystring;

    // Make sure our parent is initialized.
    Base::initialize();
}

void Derived::do_write(std::ostream &o) const
{
    // Write our data
    o << "\t" << mystring << std::endl;

    // and make sure parent writes its data.
    Base::do_write(o);
}

// The derived class overrider.  This class just reads its information
// in, then invokes the base class function.
void Derived::do_read(std::istream &i)
{
    // Read our data.
    i.ignore();
    std::getline(i, mystring);

    // and make sure the base class gets a chance to read its data.
    Base::do_read(i);
}

void Derived::displayValues()
{
    std::cout << "Hi, I'm a Derived, and my string is " << mystring;
    std::cout << "\n\tMy parent says:";
    Base::displayValues();
}

